/*
 *  Copyright 2019-2025 Carnegie Mellon University
 *  See license information in LICENSE.txt.
 */
/**
 *  @file ipfix2json.h
 *
 *  IPFIX to JSON conversion tool.
 */
/*
 *  @DISTRIBUTION_STATEMENT_BEGIN@
 *  fixbuf-tools 4.0
 *
 *  Copyright 2024 Carnegie Mellon University.
 *
 *  NO WARRANTY. THIS CARNEGIE MELLON UNIVERSITY AND SOFTWARE ENGINEERING
 *  INSTITUTE MATERIAL IS FURNISHED ON AN "AS-IS" BASIS. CARNEGIE MELLON
 *  UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR
 *  IMPLIED, AS TO ANY MATTER INCLUDING, BUT NOT LIMITED TO, WARRANTY OF
 *  FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY, OR RESULTS
 *  OBTAINED FROM USE OF THE MATERIAL. CARNEGIE MELLON UNIVERSITY DOES NOT
 *  MAKE ANY WARRANTY OF ANY KIND WITH RESPECT TO FREEDOM FROM PATENT,
 *  TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 *
 *  Licensed under a GNU GPL 2.0-style license, please see LICENSE.txt or
 *  contact permission@sei.cmu.edu for full terms.
 *
 *  [DISTRIBUTION STATEMENT A] This material has been approved for public
 *  release and unlimited distribution.  Please see Copyright notice for
 *  non-US Government use and distribution.
 *
 *  This Software includes and/or makes use of Third-Party Software each
 *  subject to its own license.
 *
 *  DM24-1024
 *  @DISTRIBUTION_STATEMENT_END@
 */

#include "fbtools-lib.h"


/* size of buffer to hold template ID and name */
#define TMPL_NAME_BUFSIZ    1024

typedef struct tmplContext_st {
    /* this is non-NULL if the template describes template-metadata */
    fbToolsTemplateName_t   *meta_template;
    /* whether this template describes element metadata (RFC5610) */
    fbToolsRfc5610_t        *meta_element;
    /* number of elements in the template */
    uint32_t                    count;
    /* memory octets required to hold record that uses this template */
    uint16_t                    len;
    /* the template ID */
    uint16_t                    tid;
} tmplContext_t;


void
idPrintTemplate(
    FILE           *fp,
    fbTemplate_t   *tmpl,
    tmplContext_t  *ctx);

void
idPrintDataRecord(
    FILE        *fp,
    fbRecord_t  *tmpl);

void
idFormatTemplateId(
    char       *tmpl_str,
    int         tid,
    const char *prefix,
    size_t      tmpl_str_bufsiz);

extern GStringChunk    *object_keys;

extern gboolean         allow_duplicate_keys;
extern fbToolsFormat_t  octet_format;
extern fbToolsFormat_t  string_format;
