/*
 *  Copyright 2014-2025 Carnegie Mellon University
 *  See license information in LICENSE.txt.
 */
/**
 *  @file ipfixDump.h
 *
 *  Prints an ipfix file in a human-readable form to outspec or stdout.
 */
/*
 *  @DISTRIBUTION_STATEMENT_BEGIN@
 *  fixbuf-tools 4.0
 *
 *  Copyright 2024 Carnegie Mellon University.
 *
 *  NO WARRANTY. THIS CARNEGIE MELLON UNIVERSITY AND SOFTWARE ENGINEERING
 *  INSTITUTE MATERIAL IS FURNISHED ON AN "AS-IS" BASIS. CARNEGIE MELLON
 *  UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR
 *  IMPLIED, AS TO ANY MATTER INCLUDING, BUT NOT LIMITED TO, WARRANTY OF
 *  FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY, OR RESULTS
 *  OBTAINED FROM USE OF THE MATERIAL. CARNEGIE MELLON UNIVERSITY DOES NOT
 *  MAKE ANY WARRANTY OF ANY KIND WITH RESPECT TO FREEDOM FROM PATENT,
 *  TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 *
 *  Licensed under a GNU GPL 2.0-style license, please see LICENSE.txt or
 *  contact permission@sei.cmu.edu for full terms.
 *
 *  [DISTRIBUTION STATEMENT A] This material has been approved for public
 *  release and unlimited distribution.  Please see Copyright notice for
 *  non-US Government use and distribution.
 *
 *  This Software includes and/or makes use of Third-Party Software each
 *  subject to its own license.
 *
 *  DM24-1024
 *  @DISTRIBUTION_STATEMENT_END@
 */

#include "fbtools-lib.h"



/* minimum width to use for an IE name */
#define MIN_IENAME_WIDTH     28

typedef struct tmplContext_st {
    /* this is non-NULL if the template describes template-metadata */
    fbToolsTemplateName_t   *meta_template;
    fbToolsRfc5610_t        *meta_element;
    /* the position of the endTime element when 'contains_end_time' is true */
    fbTemplateField_t   end_time_index;
    /* number of elements in the template */
    uint16_t            elem_count;
    /* the octet length of non-variable length elements in this template */
    uint16_t            fixed_octets;
    /* the octet length of the smallest record seen that uses this template */
    uint16_t            min_octets;
    /* the octet length of the largest record seen that uses this template */
    uint16_t            max_octets;
    /* number of records seen that use this template */
    uint32_t            rec_count;
    /* the sum of octet lengths of all records seen that use this template */
    uint64_t            all_octets;
    /* width to use when printing IE name; computed from longest name in the
     * template with a floor of MIN_IENAME_WIDTH */
    int                 iename_width;
    /* memory octets required to hold record that uses this template */
    uint16_t            len;
    /* the template ID */
    uint16_t            tid;
    /* the template scope */
    uint16_t            scope;
    /* TRUE when this template contains no varlen elements (string,list) */
    gboolean            is_fixedlen;
    /* whether this template contains the endTime element; if TRUE, the
     * 'end_time_index' member contains the location of the element */
    gboolean            contains_end_time;
} tmplContext_t;


void
idPrintTemplate(
    FILE                 *fp,
    const fbTemplate_t   *tmpl,
    const tmplContext_t  *ctx);

uint32_t
idPrintDataRecord(
    FILE                 *fp,
    tmplContext_t        *tctx,
    const fbRecord_t     *record,
    unsigned int          rec_count,
    const char           *rec_title,
    const char           *str_prefix);

void
idPrintTimestamp(
    FILE                     *fp,
    const struct timespec    *ts,
    fbInfoElementDataType_t   datatype);

void
tmplContextUpdateReclenStats(
    tmplContext_t  *tctx,
    uint32_t        reclen);

extern fbToolsFormat_t   octet_array_format;
extern fbToolsFormat_t   string_format;
extern gboolean          suppress_record_number;
